@@ -3,7 +3,7 @@ import os.path
 import pathlib
 import re
 import sys
-from typing import Optional, Set
+from typing import Dict, Optional, Set
 import uuid
 
 from bs4 import BeautifulSoup, SoupStrainer
@@ -21,7 +21,7 @@ _URL_BOT_ID = f'Bot {uuid.uuid4()}'
 URL_HEADERS = {'User-Agent': _URL_BOT_ID, 'Accept-Language': '*'}
 
 EXTERNAL_URL_PATTERN = re.compile(r'https?://')
-MARKDOWN_ANCHOR_PATTERN = re.compile(r'(.+)#(.+)')
+MARKDOWN_ANCHOR_PATTERN = re.compile(r'([^#]+)(#(.+))?')
 HEADING_PATTERN = re.compile(r'\s*#+\s*(.*)')
 HTML_LINK_PATTERN = re.compile(r'.*<a id=\"(.*)\">.*')
 IMAGE_PATTERN = re.compile(r'\[\!\[.*\]\(.*\)\].*|\!\[.*\]\[.*\].*')
@@ -34,11 +34,13 @@ urllib3.disable_warnings()
 
 
 class HtmlProoferPlugin(BasePlugin):
-    files: Files
+    files: Dict[str, File]
+    invalid_links = False
 
     config_scheme = (
         ("enabled", config_options.Type(bool, default=True)),
         ('raise_error', config_options.Type(bool, default=False)),
+        ('raise_error_after_finish', config_options.Type(bool, default=False)),
         ('raise_error_excludes', config_options.Type(dict, default={})),
         ('validate_external_urls', config_options.Type(bool, default=True)),
         ('validate_rendered_template', config_options.Type(bool, default=False)),
@@ -49,11 +51,16 @@ class HtmlProoferPlugin(BasePlugin):
         self._session.verify = False
         self._session.headers.update(URL_HEADERS)
         self._session.max_redirects = 5
+        self.files = {}
         super().__init__()
 
+    def on_post_build(self, config: Config)-> None:
+        if self.config['raise_error_after_finish'] and self.invalid_links:
+            raise PluginError("Invalid links present.")
+
     def on_page_markdown(self, markdown: str, page: Page, config: Config, files: Files) -> None:
         # Store files to allow inspecting Markdown files in later stages.
-        self.files = files
+        self.files.update({os.path.normpath(file.url): file for file in files})
 
     def on_post_page(self, output_content: str, page: Page, config: Config) -> None:
         if not self.config['enabled']:
@@ -81,7 +88,9 @@ class HtmlProoferPlugin(BasePlugin):
                 is_error = self.is_error(self.config, url, url_status)
                 if self.config['raise_error'] and is_error:
                     raise PluginError(error)
-                elif is_error:
+                elif self.config['raise_error_after_finish'] and is_error and not self.invalid_links:
+                    self.invalid_links = True
+                if is_error:
                     print(error)
 
     @lru_cache(maxsize=1000)
@@ -96,8 +105,8 @@ class HtmlProoferPlugin(BasePlugin):
         except requests.exceptions.ConnectionError:
             return -1
 
-    def get_url_status(self, url: str, src_path: str, all_element_ids: Set[str], files: Files,
-                       use_directory_urls: bool) -> int:
+    def get_url_status(self, url: str, src_path: str, all_element_ids: Set[str], files: Dict[str, File],
+        use_directory_urls: bool) -> int:
         if any(pat.match(url) for pat in LOCAL_PATTERNS):
             return 0
 
@@ -112,30 +121,53 @@ class HtmlProoferPlugin(BasePlugin):
             # Markdown file, so disable target anchor validation in this case. Examples include:
             # ../..#BAD_ANCHOR style links to index.html and extra ../ inserted into relative
             # links.
-
-            match = MARKDOWN_ANCHOR_PATTERN.match(url)
-            if match is not None:
-                # URL is a link to another local Markdown file that includes an anchor.
-                url_target, anchor = match.groups()
-                target_markdown = self.find_target_markdown(url_target, src_path, files)
-                if (target_markdown is None
-                        or not self.contains_anchor(target_markdown, anchor)):
-                    # The corresponding Markdown header for this anchor was not found.
-                    return 404
+            if not self.is_url_target_valid(url, src_path, files):
+                return 404
         return 0
 
     @staticmethod
-    def find_target_markdown(url: str, src_path: str, files: Files) -> Optional[str]:
+    def is_url_target_valid(url: str, src_path: str, files: Dict[str, File]) -> bool:
+        match = MARKDOWN_ANCHOR_PATTERN.match(url)
+        if match is None:
+            return True
+
+        url_target, _, optional_anchor = match.groups()
+        _, extension = os.path.splitext(url_target)
+        if extension == ".html":
+            # URL is a link to another local Markdown file that may includes an anchor.
+            target_markdown = HtmlProoferPlugin.find_target_markdown(url_target, src_path, files)
+            if target_markdown is None:
+                # The corresponding Markdown page was not found.
+                return False
+            if optional_anchor and not HtmlProoferPlugin.contains_anchor(target_markdown, optional_anchor):
+                # The corresponding Markdown header for this anchor was not found.
+                return False
+        elif HtmlProoferPlugin.find_source_file(url_target, src_path, files) is None:
+            return False
+
+        return True
+
+    @staticmethod
+    def find_target_markdown(url: str, src_path: str, files: Dict[str, File]) -> Optional[str]:
         """From a built URL, find the original Markdown source from the project that built it."""
 
+        file = HtmlProoferPlugin.find_source_file(url, src_path, files)
+        if file and file.page:
+            return file.page.markdown
+        return None
+
+    @staticmethod
+    def find_source_file(url: str, src_path: str, files: Dict[str, File]) -> Optional[File]:
+        """From a built URL, find the original file from the project that built it."""
+
         # Handle relative links by concatenating the source dir with the destination path
         search_path = os.path.normpath(str(pathlib.Path(src_path).parent / pathlib.Path(url)))
-        for file in files.src_paths.values():  # type: File
-            if os.path.normpath(file.url) == search_path and file.page:
-                return file.page.markdown
 
-        print(f"Warning: Unable to locate Markdown source file for: {url}", file=sys.stderr)
-        return None
+        try:
+            return files[search_path]
+        except KeyError:
+            print(f"Warning: Unable to locate source file for: {url}", file=sys.stderr)
+            return None
 
     @staticmethod
     def contains_anchor(markdown: str, anchor: str) -> bool:
